#pragma once

#include <map>
#include <mutex>
#include <sstream>

#include "region.h"
#include "types.h"



/* Memory map entry. */

typedef std::map<offset_t, Region *> _MemoryMap;
typedef std::pair<offset_t, Region *> _MemoryMapEntry;



/* Memory organised as a collection of regions. */

class MemoryMap
{
protected:
    std::mutex _lock;
    _MemoryMap _regions;

public:
    /* Region maintenance. */

    void insert(Region *region);

    void remove(Region *region);

    /* Debugging methods. */

    void show(std::ostringstream &buffer);
};



/* An allocator abstraction. */

class Allocator : public MemoryMap
{
public:
    Region *region(offset_t size);

    void release(Region *region);
};

// vim: tabstop=4 expandtab shiftwidth=4
